@@ -252,8 +252,7 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     is_cds <- type %in% .CDS_TYPES
     if (!is.null(phase)) {
         if (S4Vectors:::anyMissingOrOutside(phase[is_cds], 0L, 2L))
-            stop(wmsg("the \"phase\" metadata column must contain 0, 1, or 2, ",
-                      "for all the CDS features"))
+            warning(wmsg("some CDS phases are missing or not between 0 and 2"))
         types_with_phase <- type[!is.na(phase) & type %in% GFF_FEATURE_TYPES]
         types_with_phase <- setdiff(as.character(unique(types_with_phase)),
                                     .CDS_TYPES)
@@ -893,8 +892,7 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
         metadata <- rbind(metadata, df1)
     }
     if (!is.na(taxonomyId)) {
-        GenomeInfoDb:::.checkForAValidTaxonomyId(taxonomyId)
-        org <- GenomeInfoDb:::.lookupSpeciesFromTaxId(taxonomyId)
+        org <- GenomeInfoDb:::lookup_organism_by_tax_id(taxonomyId)
         organism <- paste(org[1,2], org[1,3])
         idx <- match("Organism", metadata$name)
         if (!is.na(idx))
